package solver;//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by FernFlower decompiler)
//

import com.gurobi.gurobi.*;
import com.gurobi.gurobi.GRB.DoubleAttr;
import com.gurobi.gurobi.GRB.DoubleParam;
import com.gurobi.gurobi.GRB.IntParam;
import net.sf.javailp.*;

import java.util.*;

public class SolverGurobi extends AbstractSolver {
    protected final Set<Hook> hooks = new HashSet();

    public SolverGurobi() {
    }

    public Result solve(Problem problem) {
        Map<Object, GRBVar> objToVar = new HashMap();
        Map<GRBVar, Object> varToObj = new HashMap();

        try {
            GRBEnv env = new GRBEnv("gurobi.log");
            this.initWithParameters(env);
            GRBModel model = new GRBModel(env);
            OptType optType = problem.getOptType();
            Map<Object, Double> optimizationCoefficients = new HashMap();
            Linear objective = problem.getObjective();
            Object variable;
            double coeff;
            if (objective != null) {
                for(Iterator i$ = objective.iterator(); i$.hasNext(); optimizationCoefficients.put(variable, coeff)) {
                    Term term = (Term)i$.next();
                    variable = term.getVariable();
                    coeff = term.getCoefficient().doubleValue();
                    if (optType == OptType.MAX) {
                        coeff *= -1.0;
                    }
                }
            }

            int i = 1;

            double primalValue;
            Iterator i$;
            for(i$ = problem.getVariables().iterator(); i$.hasNext(); ++i) {
                variable = i$.next();
                VarType varType = problem.getVarType(variable);
                Number lowerBound = problem.getVarLowerBound(variable);
                Number upperBound = problem.getVarUpperBound(variable);
                primalValue = lowerBound != null ? lowerBound.doubleValue() : -1.7976931348623157E308;
                double ub = upperBound != null ? upperBound.doubleValue() : Double.MAX_VALUE;
                String name = variable.toString();
                char type;
                switch (varType) {
                    case BOOL:
                        type = 'B';
                        break;
                    case INT:
                        type = 'I';
                        break;
                    default:
                        type = 'C';
                }

                Double coeff2 = (Double)optimizationCoefficients.get(variable);
                if (coeff2 == null) {
                    coeff2 = 0.0;
                }

                GRBVar var = model.addVar(primalValue, ub, coeff2, type, name);
                objToVar.put(variable, var);
                varToObj.put(var, variable);
            }

            model.update();

            Constraint constraint;
            GRBLinExpr expr;
            char operator;
            for(i$ = problem.getConstraints().iterator(); i$.hasNext(); model.addConstr(expr, operator, constraint.getRhs().doubleValue(), constraint.getName())) {
                constraint = (Constraint)i$.next();
                expr = new GRBLinExpr();
                Iterator j$ = constraint.getLhs().iterator();

                while(j$.hasNext()) {
                    Term term = (Term)j$.next();
                    GRBVar var = (GRBVar)objToVar.get(term.getVariable());
                    expr.addTerm(term.getCoefficient().doubleValue(), var);
                }

                if (constraint.getOperator() == Operator.GE) {
                    operator = '>';
                } else if (constraint.getOperator() == Operator.LE) {
                    operator = '<';
                } else {
                    operator = '=';
                }
            }

            i$ = this.hooks.iterator();

            while(i$.hasNext()) {
                Hook hook = (Hook)i$.next();
                hook.call(env, model, objToVar, varToObj, problem);
            }

            //The two lines below can be uncommented to investigate why a model is infeasible
            //model.computeIIS();
            //model.write("infeasible.ilp");

            model.optimize();
            System.out.println("Model status is "+ model.get(GRB.IntAttr.Status));
            System.out.println("Model runtime is "+ model.get(GRB.DoubleAttr.Runtime));

            //Below code to exit in a controlled way if the model is infeasible
            if (model.get(GRB.IntAttr.Status) == 3){//INFEASIBLE See https://www.gurobi.com/documentation/9.1/refman/optimization_status_codes.html
                return null;
            }

            ResultImpl result;
            if (problem.getObjective() != null) {
                result = new ResultImpl(problem.getObjective());
            } else {
                result = new ResultImpl();
            }

            Iterator j$ = objToVar.entrySet().iterator();

            while(j$.hasNext()) {
                Map.Entry<Object, GRBVar> entry = (Map.Entry)j$.next();
                Object variable2 = entry.getKey();
                GRBVar var = (GRBVar)entry.getValue();
                primalValue = var.get(DoubleAttr.X);
                if (problem.getVarType(variable2).isInt()) {
                    int v = (int)Math.round(primalValue);
                    result.putPrimalValue(variable2, v);
                } else {
                    result.putPrimalValue(variable2, primalValue);
                }
            }

            return result;
        } catch (GRBException var23) {
            var23.printStackTrace();
            return null;
        }
    }

    protected void initWithParameters(GRBEnv env) throws GRBException {
        Object verbose = this.parameters.get(1);
        Object timeout = this.parameters.get(0);
        Number number;
        if (verbose != null && verbose instanceof Number) {
            number = (Number)verbose;
            int value = number.intValue();
            byte msgLevel;
            switch (value) {
                case 0:
                    msgLevel = 0;
                    break;
                default:
                    msgLevel = 1;
            }

            env.set(IntParam.OutputFlag, msgLevel);
        }

        if (timeout != null && timeout instanceof Number) {
            number = (Number)timeout;
            double value = number.doubleValue();
            env.set(DoubleParam.TimeLimit, value);
        }

    }

    public void addHook(Hook hook) {
        this.hooks.add(hook);
    }

    public void removeHook(Hook hook) {
        this.hooks.remove(hook);
    }

    public interface Hook {
        void call(GRBEnv var1, GRBModel var2, Map<Object, GRBVar> var3, Map<GRBVar, Object> var4, Problem var5);
    }
}
